#!/usr/bin/env python

#    This file is part of VASM.
#
#    VASM is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License v3 as published by
#    the Free Software Foundation.
#
#    VASM is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VASM.  If not, see <http://www.gnu.org/licenses/>.

__author__ = "Moises Henriquez"
__author_email__ = "moc.liamg@xnl.E0M"[::-1]

import shutil
from string import capwords
import logging

log = logging.getLogger('vasm')

""" REPOS.py
provides methods for inteacting with /etc/slapt-get/slapt-getrc"""

class SlaptgetRcModel(object):
    """ data model for working with a slapt-getrc file """
    def __init__(self, filepath="/etc/slapt-get/slapt-getrc"):
        self.filepath = filepath
        self.observers = []
    
    def add_observer(self, method):
        """ Add an observer to the notification list """
        self.observers.append(method)
        return

    def notify(self):
        for method in self.observers:
            method()
    
    def save_changes(self, changedlist):
        """Save the changes to the actual file.
        Arguments:
            changedlist = a list of tuples indicating the following
                (True, repo, priority)
                where True means Enabled and False means Disabled.
                
        Returns False when successful.
        """
        newdata = []
        f = open(self.filepath, "r")
        fdata = f.readlines()
        f.close()
        for line in fdata:
            if line.startswith("EXCLUDE=") or line.startswith("WORKINGDIR"):
                newdata.append(line)
            else:
                continue
        for entry in changedlist:
            status = entry[0]
            url = entry[1]
            priority = entry[2]
            if status == True:
                state = "SOURCE"
            else:
                state = "#DISABLED"
            if priority == "Default":
                priotext = ""
            else:
                priotext = ":%s"% priority.upper()
            line = "%s=%s%s\n"% (state, url, priotext)
            newdata.append(line)
            
        # save a copy of the old file
        shutil.copy(self.filepath, "%s.bak"%self.filepath)
        # Write to the disk, DO NOT catch this exception
        f = open(self.filepath, 'w')
        f.writelines(''.join(newdata))
        f.close()
    
    def get_current_active_repos(self):
        """ Returns the list of currently active repos """
        ret = []
        f = open(self.filepath, "r")
        data = f.readlines()
        f.close()
        for line in data:
            if line.startswith("SOURCE="):
                content = line.split("=")[-1]
                if content.count(":") > 1:
                    url = content.rsplit(":", 1)[0].strip()
                    pri = capwords(content.rsplit(':',1)[-1])
                else:
                    url = content.strip()
                    pri = "Default"
                ret.append((url, pri))
        return ret

    def get_current_disabled_repos(self):
        """ Returns a list of disabled repos """
        ret = []
        f = open(self.filepath, "r")
        data = f.readlines()
        f.close()
        for line in data:
            if line.startswith("#DISABLED") or line.startswith("#SOURCE"):
                content = line.split("=")[-1]
                if content.count(":") > 1:
                    url = content.rsplit(":", 1)[0].strip()
                    pri = capwords(content.rsplit(":",1)[-1])
                else:
                    url = content.strip()
                    pri = "Default"
                ret.append((url, pri))
        return ret
    
