#!/usr/bin/env python

#    This file is part of VASM.
#
#    VASM is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License v3 as published by
#    the Free Software Foundation.
#
#    VASM is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VASM.  If not, see <http://www.gnu.org/licenses/>.

__author__ = "Moises Henriquez"
__author_email__ = "moc.liamg@xnl.E0M"[::-1]

""" Display basic information about the vectorlinux system """

import os
from vasm.backend.utils import _, datapath, get_system_info
from vasm.ui.gtk2.support import widgets, dialogs
import gtk

# VASM module constants
VASM_CATEGORY=_("System")
VASM_LABEL=_("About VectorLinux")
VASM_ICON = os.path.join(datapath, 'vectorlinux-logo.png')

class AboutVL(widgets.VasmModule):
    def __init__(self, parent):
        title = VASM_LABEL
        widgets.VasmModule.__init__(self, parent, title, "")
        
        # the system information
        info = get_system_info()
        
        # the logo
        logopath = os.path.join(datapath, 'vectorlinux-logo.png')
        logo_pb = gtk.gdk.pixbuf_new_from_file_at_size(logopath, 96, 96)
        logo_img = gtk.Image()
        logo_img.set_from_pixbuf(logo_pb)
        logobox = gtk.HBox()
        logobox.pack_start(logo_img, True, False, 4)
        self.pack_start(logobox, False, False, 4)
        
        # The system details
        bannerbox = gtk.HBox()
        banner = gtk.Label()
        bannerbox.pack_start(banner, True, True, 2)
        banner.set_markup("<b>VectorLinux %s</b>"% (
            info["version"]
        ))
        self.pack_start(bannerbox, False, False, 4)
        #lbwebsite = gtk.Label("http://www.vectorlinux.com")
        lbwebsite = gtk.LinkButton("http://www.vectorlinux.com", "http://www.vectorlinux.com")
        lbwebsite.set_property('relief', gtk.RELIEF_NONE)
        lbrelease = gtk.Label("%s: %s"% (_("Release Date"), info["releasedate"]))
        lbbase = gtk.Label("%s: %s"% (_("System Base"), info["base"]))
        lbhostname = gtk.Label("%s: %s"% (_("Host name"), os.uname()[1]))
        lbvmlinuz = gtk.Label("%s: %s"% (_("Running Kernel Version"), os.uname()[2]))
        lbarch = gtk.Label("%s: %s"% (_("System Architecture"), os.uname()[-1]))
        
        for lb in (lbwebsite, lbrelease, lbbase, lbhostname, lbvmlinuz, lbarch):
            if type(lb) == gtk.Label:
                lb.set_property('xalign', 0.0)
            box = gtk.HBox()
            box.pack_start(lb, False, False, 2)
            self.pack_start(box, False, False, 2)
        # buttons
        btclose = gtk.Button(stock=gtk.STOCK_CLOSE)
        btclose.connect('clicked', self.close_module)
        self.add_button(btclose)

def __vasm_test__():
    """ This module should be available to everyone """
    return True

def __run__(parent=None):
    about = AboutVL(parent)
    return about.launch()

