#!/usr/bin/env python

#    This file is part of VASM.
#
#    VASM is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License v3 as published by
#    the Free Software Foundation.
#
#    VASM is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VASM.  If not, see <http://www.gnu.org/licenses/>.

import gtk
import os
from vasm.backend.utils import datapath, __vasm_version__

__author__ = "Moises Henriquez"
__author_email__ = "moc.liamg@xnl.E0M"[::-1]

class AboutVASM(gtk.AboutDialog):
    """ About window for vasm """
    def __init__(self, parent):
        gtk.AboutDialog.__init__(self)
        self.set_name("VASM")
        self.set_version(__vasm_version__)
        self.set_copyright("Copyright 2011 Moises Henriquez")
        f = open(os.path.join(datapath,'LICENSE'),'r')
        license = f.readlines()
        f.close()
        self.set_license("".join(license))
        self.set_logo(
            gtk.gdk.pixbuf_new_from_file_at_size(
                os.path.join(datapath,'vasmcc-logo.png'), 200,200
            )
        )
        self.set_authors(["Moises Henriquez [ %s ]"% "moc.liamg@xnl.E0M"[::-1],
                        "Jason Pierce [ %s ]"% "moc.liamg@nosaj.ecreip"[::-1],
                        "Rodrigo Bistolfi [ %s ]"% "moc.liamg@iflotsibr"[::-1],
                        "Uel Archuletta [ %s ]" % "moc.liamg@s8ksleU"[::-1],])
        self.set_website("https://bitbucket.org/m0e_lnx/vasm")
        self.set_comments("VectorLinux control panel utility.")
        self.set_icon(parent.get_icon())

def info(txt, title="Info", parent=None):
    dia=gtk.MessageDialog(parent=parent,
                          type=gtk.MESSAGE_INFO,
                          buttons=gtk.BUTTONS_OK)
    dia.set_markup(txt)
    dia.set_title(title)
    dia.set_position(gtk.WIN_POS_CENTER)
    ret = dia.run()
    dia.destroy()
    return ret

def error(txt, title="Error", parent=None):
    dia = gtk.MessageDialog(parent=parent,
                            type=gtk.MESSAGE_ERROR,
                            buttons=gtk.BUTTONS_OK)
    dia.set_markup(txt)
    dia.set_title(title)
    dia.set_position(gtk.WIN_POS_CENTER)
    ret = dia.run()
    dia.destroy()
    return ret

def question(txt, title="Question", parent=None):
    dia = gtk.MessageDialog(parent=parent,
                            type=gtk.MESSAGE_QUESTION,
                            buttons=gtk.BUTTONS_YES_NO)
    dia.set_markup(txt)
    dia.set_title(title)
    dia.set_position(gtk.WIN_POS_CENTER)
    ret = dia.run()
    dia.destroy()
    return ret

def warning(txt, title="Warning!", parent=None):
    dia = gtk.MessageDialog(parent=parent,
                            type=gtk.MESSAGE_WARNING,
                            buttons=gtk.BUTTONS_OK)
    dia.set_position(gtk.WIN_POS_CENTER)
    dia.set_title(title)
    dia.set_markup(txt)
    ret = dia.run()
    dia.destroy()
    return ret
