#!/usr/bin/env python

#    This file is part of VASM.
#
#    VASM is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License v3 as published by
#    the Free Software Foundation.
#
#    VASM is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VASM.  If not, see <http://www.gnu.org/licenses/>.

""" aboutvl.py
Text mode module to show information about the current vectorlinux installation
"""
__author__ = "Moises Henriquez"
__author_email__ = "moc.liamg@xnl.E0M"[::-1]


import urwid
import sys
import os
from vasm.backend import utils
from vasm.ui.tui.support import dialogs, widgets

_=utils._

VASM_CATEGORY = "System"
VASM_LABEL = "About VectorLinux"


class AboutVL(widgets.TuiModule):
    """ Show dialog about vl """
    def __init__(self, parent):
        self.parent = parent
        info = {
            "VectorLinux Version: ": utils.get_vector_version(),
            "System Base: ": utils.get_slack_base(),
            "Release Date: ": utils.get_release_date(),
            "Running Kernel Version: ": os.uname()[2],
            "System Architecture: ": os.uname()[-1],
            "Hostname: ": os.uname()[1],
      }
        body = []
        body.append(widgets.blank)
        for item in info:
           wrap = urwid.AttrWrap(
               urwid.Text("%s %s"% ( item, info[item])), 'body'
           )
           body.append(wrap)
        body.append(urwid.Divider())
        body.append(urwid.AttrWrap(
            urwid.Text("www.vectorlinux.com"), 'body'))
        widgets.TuiModule.__init__(self,
                                      header="About VectorLinux",
                                      desc = ["",_("Information about your vectorlinux system.")],
                                      body = body,
                                      buttons = [("Back", self.close_module)],
                                      parent=self.parent)
        
        self.body = urwid.LineBox(self.get_body())
    
    def return_to_module(self, widget=None, data=None):
        """ REturn to this module from another dialog.
        
        Modules that do not have child dialogs, may just reuturn return_home"""
        return self.return_home(widget)

def __vasm_test__():
    return True


def __run__(parent_toplevel):
    mod = AboutVL(parent_toplevel)
    parent_toplevel.pop_module(mod)
    
    
